@@ -39,6 +39,7 @@ HD_DEFINE_PLUGIN_MODULE(DesktopSwitcherHomePlugin, desktop_switcher_home_plugin,
 #define DESKTOP_SWITCHER_HOME_PLUGIN_SETTINGS_FOLDER ".desktop_switcher"
 #define DESKTOP_SWITCHER_CONFIG_KEY_WIDTH "width"
 #define DESKTOP_SWITCHER_CONFIG_KEY_HEIGHT "height"
+#define DESKTOP_SWITCHER_CONFIG_KEY_ALPHA "alpha"
 #define DESKTOP_SWITCHER_CONFIG_KEY_SWITCH_TO "switch_to"
 #define DESKTOP_SWITCHER_CONFIG_KEY_ICON_PATH "icon_path"
 #define DESKTOP_SWITCHER_CONFIG_KEY_SCROLL_TYPE "scroll_type"
@@ -60,13 +61,16 @@ HD_DEFINE_PLUGIN_MODULE(DesktopSwitcherHomePlugin, desktop_switcher_home_plugin,
 struct _DesktopSwitcherHomePluginPrivate
 {
   gchar* iD;
-  gint width;
-  gint height;
+  guint width;
+  guint height;
+  guint alpha;
   gint switch_to;
   gboolean smooth;
   gchar* icon_path;
   GdkPixbuf* background_pixbuf;
-
+  guint icon_width;
+  guint icon_height;
+  gboolean pressed;
 };
 
 typedef struct _swipe_data_t
@@ -274,6 +278,16 @@ _on_button_release(GtkWidget* widget, GdkEventButton* event, gpointer user_data)
     else
       _switch_desktop_right(dpy, hops, desktop_plugin->priv->smooth);
   }
+  gtk_widget_queue_draw(GTK_WIDGET(desktop_plugin));
+  return TRUE;
+}
+
+static gboolean
+_on_button_press(GtkWidget* widget, GdkEventButton* event, gpointer user_data)
+{
+  DesktopSwitcherHomePlugin* desktop_plugin = DESKTOP_SWITCHER_HOME_PLUGIN(user_data);
+  desktop_plugin->priv->pressed = TRUE;
+  gtk_widget_queue_draw(GTK_WIDGET(desktop_plugin));
   return TRUE;
 }
 
@@ -284,6 +298,17 @@ _switcher_load_icon(DesktopSwitcherHomePlugin* desktop_plugin)
     g_object_unref(desktop_plugin->priv->background_pixbuf);
   
   desktop_plugin->priv->background_pixbuf = gdk_pixbuf_new_from_file(desktop_plugin->priv->icon_path, NULL);
+  if(desktop_plugin->priv->background_pixbuf)
+  {
+
+    desktop_plugin->priv->icon_width = gdk_pixbuf_get_width(desktop_plugin->priv->background_pixbuf);
+    desktop_plugin->priv->icon_height = gdk_pixbuf_get_height(desktop_plugin->priv->background_pixbuf);
+  }
+  else
+  {
+    desktop_plugin->priv->icon_width = 0;
+    desktop_plugin->priv->icon_height = 0;
+  }
 }
 
 static gboolean
@@ -297,9 +322,24 @@ desktop_switcher_expose(GtkWidget* widget, GdkEventExpose* event)
   gdk_cairo_region(cr, event->region);
   cairo_clip(cr);
   cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+  if(desktop_plugin->priv->width != desktop_plugin->priv->icon_width ||
+     desktop_plugin->priv->height != desktop_plugin->priv->icon_height)
+  {
+    cairo_scale(cr, 
+		(double)desktop_plugin->priv->width/desktop_plugin->priv->icon_width,
+		(double)desktop_plugin->priv->height/desktop_plugin->priv->icon_height);
+  }
+  if(desktop_plugin->priv->pressed)
+  {
+    cairo_scale(cr, 0.90, 0.90);
+    cairo_translate(cr, 
+		    (double)desktop_plugin->priv->icon_width*0.05,
+		    (double)desktop_plugin->priv->icon_height*0.05);
+    desktop_plugin->priv->pressed = FALSE;
+  }
   if(desktop_plugin->priv->background_pixbuf)
     gdk_cairo_set_source_pixbuf(cr, desktop_plugin->priv->background_pixbuf, 0, 0);
-  cairo_paint(cr);
+  cairo_paint_with_alpha(cr, (256-desktop_plugin->priv->alpha)/256.0);
   cairo_destroy(cr);
   return GTK_WIDGET_CLASS(desktop_switcher_home_plugin_parent_class)->expose_event(widget, event);
 }
@@ -336,6 +376,15 @@ _read_settings(DesktopSwitcherHomePlugin* desktop_plugin)
 							    DESKTOP_SWITCHER_CONFIG_KEY_HEIGHT, 
 							    NULL);
     }
+    if(g_key_file_has_key(key_file, desktop_plugin->priv->iD, 
+			  DESKTOP_SWITCHER_CONFIG_KEY_ALPHA, 
+			  NULL))
+    {
+      desktop_plugin->priv->alpha = g_key_file_get_integer(key_file,
+							   desktop_plugin->priv->iD,
+							   DESKTOP_SWITCHER_CONFIG_KEY_ALPHA, 
+							   NULL);
+    }
     if(g_key_file_has_key(key_file, desktop_plugin->priv->iD, 
 			  DESKTOP_SWITCHER_CONFIG_KEY_SWITCH_TO, 
 			  NULL))
@@ -406,6 +455,10 @@ _write_settings(DesktopSwitcherHomePlugin* desktop_plugin)
 			 desktop_plugin->priv->iD,
 			 DESKTOP_SWITCHER_CONFIG_KEY_HEIGHT, 
 			 desktop_plugin->priv->height);
+  g_key_file_set_integer(key_file,
+			 desktop_plugin->priv->iD,
+			 DESKTOP_SWITCHER_CONFIG_KEY_ALPHA, 
+			 desktop_plugin->priv->alpha);
   g_key_file_set_integer(key_file,
 			 desktop_plugin->priv->iD,
 			 DESKTOP_SWITCHER_CONFIG_KEY_SWITCH_TO, 
@@ -444,6 +497,13 @@ desktop_switcher_realize(GtkWidget* widget)
   gtk_widget_set_app_paintable(widget, TRUE);
   desktop_plugin->priv->iD = hd_home_plugin_item_get_applet_id(HD_HOME_PLUGIN_ITEM(widget));
   _read_settings(desktop_plugin);
+  gtk_widget_set_size_request(GTK_WIDGET(desktop_plugin), 
+			      desktop_plugin->priv->width,
+			      desktop_plugin->priv->height);
+  gtk_window_resize(GTK_WINDOW(desktop_plugin), 
+		    desktop_plugin->priv->width,
+		    desktop_plugin->priv->height);
+
   GTK_WIDGET_CLASS(desktop_switcher_home_plugin_parent_class)->realize(widget);
 }
 
@@ -471,13 +531,13 @@ _create_desktop_list(DesktopSwitcherHomePlugin* desktop_plugin, int* row_for_thi
 
   gtk_list_store_append(list_store, &iter);
   gtk_list_store_set(list_store, &iter,
-		     0, g_strdup("left"), 
+		     0, "left", 
 		     1, SWITCH_TO_LEFT,
 		     -1);
   index++;
   gtk_list_store_append(list_store, &iter);
   gtk_list_store_set(list_store, &iter,
-		     0, g_strdup("right"), 
+		     0, "right", 
 		     1, SWITCH_TO_RIGHT,
 		     -1);
   index++;
@@ -488,10 +548,12 @@ _create_desktop_list(DesktopSwitcherHomePlugin* desktop_plugin, int* row_for_thi
   {
     int view_number = GPOINTER_TO_INT(active_view->data);
     gtk_list_store_append(list_store, &iter);
+    gchar* view_number_c = g_strdup_printf("%d", view_number); 
     gtk_list_store_set(list_store, &iter,
-		       0, g_strdup_printf("%d", view_number), 
+		       0, view_number_c,
 		       1, view_number,
 		       -1);
+    g_free(view_number_c);
     if(view_number == desktop_plugin->priv->switch_to)
       *row_for_this_desktop = index;
     index++;
@@ -502,6 +564,20 @@ _create_desktop_list(DesktopSwitcherHomePlugin* desktop_plugin, int* row_for_thi
   return list_store;
 }
 
+static void
+_increment_slider(GtkButton* button, gpointer user_data)
+{
+  GtkRange* range = GTK_RANGE(user_data);
+  gtk_range_set_value(range, gtk_range_get_value(range) + 1);
+}
+
+static void
+_decrement_slider(GtkButton* button, gpointer user_data)
+{
+  GtkRange* range = GTK_RANGE(user_data);
+  gtk_range_set_value(range, gtk_range_get_value(range) - 1);
+}
+
 static void
 _show_settings_dialog(GtkWidget* widget, gpointer data)
 {
@@ -510,6 +586,13 @@ _show_settings_dialog(GtkWidget* widget, gpointer data)
   GtkWidget* switch_to_selector_button;
   GtkWidget* fast_scroll_toggle_button;
   GtkWidget* smooth_scroll_toggle_button;
+  GtkWidget* width_slider;
+  GtkWidget* height_slider;
+  GtkWidget* alpha_slider;
+
+  GtkWidget* wlabel = gtk_label_new("Width:");
+  GtkWidget* hlabel = gtk_label_new("Height:");
+  GtkWidget* alabel = gtk_label_new("Transparency:");
 
   dialog = gtk_dialog_new_with_buttons("Settings",
 				       NULL,
@@ -541,11 +624,93 @@ _show_settings_dialog(GtkWidget* widget, gpointer data)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(smooth_scroll_toggle_button), TRUE);
   else
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fast_scroll_toggle_button), TRUE);
+
+  width_slider = hildon_gtk_hscale_new();
+  gtk_adjustment_configure(gtk_range_get_adjustment(GTK_RANGE(width_slider)),
+			   desktop_plugin->priv->width,
+			   32, 400,
+			   1, 1, 1);
+
+  gtk_scale_set_value_pos(GTK_SCALE(width_slider), GTK_POS_RIGHT);
+  gtk_scale_set_draw_value(GTK_SCALE(width_slider), TRUE);
+  gtk_scale_set_digits(GTK_SCALE(width_slider), 0);
+
+  height_slider = hildon_gtk_hscale_new();
+  gtk_adjustment_configure(gtk_range_get_adjustment(GTK_RANGE(height_slider)),
+			   desktop_plugin->priv->height,
+			   32, 420,
+			   1, 1, 1);
+  gtk_scale_set_value_pos(GTK_SCALE(height_slider), GTK_POS_RIGHT);
+  gtk_scale_set_draw_value(GTK_SCALE(height_slider), TRUE);
+  gtk_scale_set_digits(GTK_SCALE(height_slider), 0);
+
+  alpha_slider = hildon_gtk_hscale_new();
+  gtk_adjustment_configure(gtk_range_get_adjustment(GTK_RANGE(alpha_slider)),
+			   desktop_plugin->priv->alpha,
+			   0, 256,
+			   1, 1, 1);
+  gtk_scale_set_value_pos(GTK_SCALE(alpha_slider), GTK_POS_RIGHT);
+  gtk_scale_set_draw_value(GTK_SCALE(alpha_slider), TRUE);
+  gtk_scale_set_digits(GTK_SCALE(alpha_slider), 0);
+
+  GtkWidget* decrement_width_button = hildon_gtk_button_new(HILDON_SIZE_FINGER_HEIGHT);
+  gtk_button_set_label(GTK_BUTTON(decrement_width_button), "-");
+  GtkWidget* increment_width_button = hildon_gtk_button_new(HILDON_SIZE_FINGER_HEIGHT);
+  gtk_button_set_label(GTK_BUTTON(increment_width_button), "+");
+  GtkWidget* decrement_height_button = hildon_gtk_button_new(HILDON_SIZE_FINGER_HEIGHT);
+  gtk_button_set_label(GTK_BUTTON(decrement_height_button), "-");
+  GtkWidget* increment_height_button = hildon_gtk_button_new(HILDON_SIZE_FINGER_HEIGHT);
+  gtk_button_set_label(GTK_BUTTON(increment_height_button), "+");
+  GtkWidget* decrement_alpha_button = hildon_gtk_button_new(HILDON_SIZE_FINGER_HEIGHT);
+  gtk_button_set_label(GTK_BUTTON(decrement_alpha_button), "-");
+  GtkWidget* increment_alpha_button = hildon_gtk_button_new(HILDON_SIZE_FINGER_HEIGHT);
+  gtk_button_set_label(GTK_BUTTON(increment_alpha_button), "+");
+
+  g_signal_connect(decrement_width_button, "clicked", G_CALLBACK(_decrement_slider), width_slider);
+  g_signal_connect(increment_width_button, "clicked", G_CALLBACK(_increment_slider), width_slider);
+  g_signal_connect(decrement_height_button, "clicked", G_CALLBACK(_decrement_slider), height_slider);
+  g_signal_connect(increment_height_button, "clicked", G_CALLBACK(_increment_slider), height_slider);
+  g_signal_connect(decrement_alpha_button, "clicked", G_CALLBACK(_decrement_slider), alpha_slider);
+  g_signal_connect(increment_alpha_button, "clicked", G_CALLBACK(_increment_slider), alpha_slider);
+
+  
   GtkWidget* scroll_button_box = gtk_hbox_new(TRUE, 3);
   gtk_box_pack_start(GTK_BOX(scroll_button_box), fast_scroll_toggle_button, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(scroll_button_box), smooth_scroll_toggle_button, TRUE, TRUE, 0);
+
+  GtkWidget* size_slider_button_box_width = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(size_slider_button_box_width), decrement_width_button, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(size_slider_button_box_width), width_slider, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(size_slider_button_box_width), increment_width_button, FALSE, FALSE, 0);
+  
+  GtkWidget* size_slider_button_box_height = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(size_slider_button_box_height), decrement_height_button, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(size_slider_button_box_height), height_slider, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(size_slider_button_box_height), increment_height_button, FALSE, FALSE, 0);
+  
+  GtkWidget* alpha_slider_button_box = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(alpha_slider_button_box), decrement_alpha_button, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(alpha_slider_button_box), alpha_slider, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(alpha_slider_button_box), increment_alpha_button, FALSE, FALSE, 0);
+
+  GtkWidget* slider_label_box = gtk_vbox_new(TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(slider_label_box), wlabel, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(slider_label_box), hlabel, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(slider_label_box), alabel, TRUE, TRUE, 0);
+
+  GtkWidget* slider_size_box = gtk_vbox_new(TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(slider_size_box), size_slider_button_box_width, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(slider_size_box), size_slider_button_box_height, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(slider_size_box), alpha_slider_button_box, TRUE, TRUE, 0);
+
+  GtkWidget* slider_box = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(slider_box), slider_label_box, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(slider_box), slider_size_box, TRUE, TRUE, 0);
+
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), switch_to_selector_button, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), scroll_button_box, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), slider_box, TRUE, TRUE, 0);
+
   gtk_widget_show_all(dialog);
 
   if(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
@@ -608,6 +773,27 @@ _show_settings_dialog(GtkWidget* widget, gpointer data)
       else
 	desktop_plugin->priv->smooth = FALSE;
     }
+
+    int new_width = gtk_range_get_value(GTK_RANGE(width_slider));
+    int new_height = gtk_range_get_value(GTK_RANGE(height_slider));
+    gboolean resize = new_width!=desktop_plugin->priv->width ||
+      new_height!=desktop_plugin->priv->height;
+
+    desktop_plugin->priv->width = new_width;
+    desktop_plugin->priv->height = new_height;
+    desktop_plugin->priv->alpha= gtk_range_get_value(GTK_RANGE(alpha_slider));
+    if(resize)
+    {
+      gtk_widget_set_size_request(GTK_WIDGET(desktop_plugin), 
+				  desktop_plugin->priv->width,
+				  desktop_plugin->priv->height);
+				  
+      gtk_window_resize(GTK_WINDOW(desktop_plugin), 
+			desktop_plugin->priv->width,
+			desktop_plugin->priv->height);
+    }
+
+
     _write_settings(desktop_plugin);
   }
   g_object_unref(desktop_list);
@@ -615,26 +801,49 @@ _show_settings_dialog(GtkWidget* widget, gpointer data)
   gtk_widget_queue_draw(GTK_WIDGET(desktop_plugin));
 }
 
+static void
+_on_current_desktop(GtkWidget* widget,
+		    gpointer data)
+{
+  DesktopSwitcherHomePlugin* desktop_plugin = DESKTOP_SWITCHER_HOME_PLUGIN(widget);
+  gboolean on;
+  g_object_get(widget, "is-on-current-desktop", &on, NULL);
+  if(on)
+  {
+    desktop_plugin->priv->pressed = FALSE;
+    gtk_widget_queue_draw(widget);
+  }
+}
+
 static void
 desktop_switcher_home_plugin_init(DesktopSwitcherHomePlugin* desktop_plugin)
 {
   desktop_plugin->priv = DESKTOP_SWITCHER_HOME_PLUGIN_GET_PRIVATE(desktop_plugin);
   desktop_plugin->priv->width = DEFAULT_WIDGET_WIDTH;
   desktop_plugin->priv->height = DEFAULT_WIDGET_HEIGHT;
+  desktop_plugin->priv->alpha = 0;
   desktop_plugin->priv->switch_to = SWITCH_TO_LEFT;
   desktop_plugin->priv->smooth = TRUE;
   desktop_plugin->priv->icon_path = g_strdup(DEFAULT_LEFT_ICON_PATH);
   desktop_plugin->priv->background_pixbuf = NULL;
+  desktop_plugin->priv->icon_width = 0;
+  desktop_plugin->priv->icon_height = 0;
+  desktop_plugin->priv->pressed = FALSE;
 
   GtkWidget* contents = gtk_event_box_new();
   gtk_event_box_set_visible_window(GTK_EVENT_BOX(contents), FALSE);
   gtk_container_set_border_width(GTK_CONTAINER(contents), 0);
+  g_signal_connect(GTK_CONTAINER(contents),
+		   "button-press-event",
+		   G_CALLBACK(_on_button_press),
+		   desktop_plugin);
   g_signal_connect(GTK_CONTAINER(contents),
 		   "button-release-event",
 		   G_CALLBACK(_on_button_release),
 		   desktop_plugin);
   hd_home_plugin_item_set_settings(HD_HOME_PLUGIN_ITEM (desktop_plugin), TRUE);
   g_signal_connect(desktop_plugin, "show-settings", G_CALLBACK(_show_settings_dialog), NULL);
+  g_signal_connect(desktop_plugin, "notify::is-on-current-desktop", G_CALLBACK(_on_current_desktop), NULL);
 
   gtk_widget_show_all(contents);
   gtk_widget_set_size_request(GTK_WIDGET(desktop_plugin),
